@@ -10,6 +10,7 @@ from shutil import which
 from typing import List
 
 from pipx import constants
+from pipx.commands.common import package_name_from_spec
 from pipx.constants import TEMP_VENV_EXPIRATION_THRESHOLD_DAYS
 from pipx.emojies import hazard
 from pipx.util import (
@@ -60,7 +61,7 @@ def run(
 
     if WINDOWS and not app.endswith(".exe"):
         app = f"{app}.exe"
-        logging.warning(f"Assuming app is {app!r} (Windows only)")
+        logging.info(f"Assuming app is {app!r} (Windows only)")
 
     pypackage_bin_path = get_pypackage_bin_path(app)
     if pypackage_bin_path.exists():
@@ -115,12 +116,15 @@ def _download_and_run(
     venv = Venv(venv_dir, python=python, verbose=verbose)
     venv.create_venv(venv_args, pip_args)
 
-    # venv.pipx_metadata.main_package.package contains package name if it is
-    #   pre-existing, otherwise is None to instruct venv.install_package to
-    #   determine package name.
+    if venv.pipx_metadata.main_package.package is not None:
+        package = venv.pipx_metadata.main_package.package
+    else:
+        package = package_name_from_spec(
+            package_or_url, python, pip_args=pip_args, verbose=verbose
+        )
 
     venv.install_package(
-        package=venv.pipx_metadata.main_package.package,
+        package=package,
         package_or_url=package_or_url,
         pip_args=pip_args,
         include_dependencies=False,
