@@ -10,9 +10,10 @@ import logging
 import re
 import textwrap
 from pathlib import Path
-from typing import List, NamedTuple, Optional, Set, Tuple
+from typing import List, NamedTuple, Optional, Tuple
 
 from packaging.requirements import InvalidRequirement, Requirement
+from packaging.specifiers import SpecifierSet
 from packaging.utils import canonicalize_name
 
 from pipx.emojies import hazard
@@ -38,7 +39,7 @@ def _split_path_extras(package_spec: str) -> Tuple[str, str]:
 def _parse_specifier(package_spec: str) -> ParsedPackage:
     """Parse package_spec as would be given to pipx
     """
-    # NOTE: If package_spec is valid pypi name, pip will always treat it as a
+    # If package_spec is valid pypi name, pip will always treat it as a
     #       pypi package, not checking for local path.
     #       We replicate pypi precedence here (only non-valid-pypi names
     #       initiate check for local path, e.g. './package-name')
@@ -55,7 +56,7 @@ def _parse_specifier(package_spec: str) -> ParsedPackage:
         # valid PEP508 package specification
         valid_pep508 = package_req
 
-    # NOTE: packaging currently (2020-07-19) only does basic syntax checks on URL.
+    # packaging currently (2020-07-19) only does basic syntax checks on URL.
     #   Some examples of what it will not catch:
     #       - invalid RCS string (e.g. "gat+https://...")
     #       - non-existent scheme (e.g. "zzzzz://...")
@@ -90,15 +91,41 @@ def _parse_specifier(package_spec: str) -> ParsedPackage:
     )
 
 
-def _extras_to_str(extras: Set):
-    if extras:
-        return "[" + ",".join(sorted(extras)) + "]"
-    else:
-        return ""
+def package_or_url_from_pep508(
+    requirement: Requirement, remove_version_specifiers=False
+) -> str:
+    requirement.marker = None
+    requirement.name = canonicalize_name(requirement.name)
+    if remove_version_specifiers:
+        requirement.specifier = SpecifierSet("")
+    return str(requirement)
 
 
-def package_is_local_path(package_spec: str) -> bool:
-    return _parse_specifier(package_spec).valid_local_path is not None
+def _parsed_package_to_package_or_url(
+    parsed_package: ParsedPackage, remove_version_specifiers: bool
+) -> str:
+    if parsed_package.valid_pep508 is not None:
+        if parsed_package.valid_pep508.marker is not None:
+            logging.warning(
+                textwrap.fill(
+                    f"{hazard}  Ignoring environment markers "
+                    f"({parsed_package.valid_pep508.marker}) in package "
+                    "specification. Use pipx options to specify this type of "
+                    "information.",
+                    subsequent_indent="    ",
+                )
+            )
+        package_or_url = package_or_url_from_pep508(
+            parsed_package.valid_pep508,
+            remove_version_specifiers=remove_version_specifiers,
+        )
+    elif parsed_package.valid_url is not None:
+        package_or_url = parsed_package.valid_url
+    elif parsed_package.valid_local_path is not None:
+        package_or_url = parsed_package.valid_local_path
+
+    logging.info(f"cleaned package spec: {package_or_url}")
+    return package_or_url
 
 
 def parse_specifier_for_install(
@@ -107,38 +134,14 @@ def parse_specifier_for_install(
     """Return package_or_url and pip_args suitable for pip install
 
     Specifically:
-    * Strip any markers (e.g. python_version > 3.4)
+    * Strip any markers (e.g. python_version > "3.4")
     * Ensure --editable is removed for any package_spec not a local path
     * Convert local paths to absolute paths
     """
     parsed_package = _parse_specifier(package_spec)
-    if parsed_package.valid_pep508 is not None:
-        if parsed_package.valid_pep508.url:
-            package_or_url = parsed_package.valid_pep508.url
-        else:
-            package_or_url = canonicalize_name(parsed_package.valid_pep508.name)
-            package_or_url += _extras_to_str(parsed_package.valid_pep508.extras)
-            if parsed_package.valid_pep508.specifier:
-                package_or_url = package_or_url + str(
-                    parsed_package.valid_pep508.specifier
-                )
-            if parsed_package.valid_pep508.marker:
-                logging.warning(
-                    textwrap.fill(
-                        f"{hazard}  Ignoring environment markers "
-                        f"({parsed_package.valid_pep508.marker}) in package "
-                        "specification. Use pipx options to specify this type of "
-                        "information.",
-                        subsequent_indent="    ",
-                    )
-                )
-    elif parsed_package.valid_url is not None:
-        package_or_url = parsed_package.valid_url
-    elif parsed_package.valid_local_path is not None:
-        package_or_url = parsed_package.valid_local_path
-
-    logging.info(f"cleaned package spec: {package_or_url}")
-
+    package_or_url = _parsed_package_to_package_or_url(
+        parsed_package, remove_version_specifiers=False
+    )
     if "--editable" in pip_args and not parsed_package.valid_local_path:
         logging.warning(
             textwrap.fill(
@@ -162,19 +165,9 @@ def parse_specifier_for_metadata(package_spec: str) -> str:
     * Convert local paths to absolute paths
     """
     parsed_package = _parse_specifier(package_spec)
-    if parsed_package.valid_pep508 is not None:
-        if parsed_package.valid_pep508.url:
-            package_or_url = parsed_package.valid_pep508.url
-        else:
-            package_or_url = canonicalize_name(parsed_package.valid_pep508.name)
-            package_or_url += _extras_to_str(parsed_package.valid_pep508.extras)
-    elif parsed_package.valid_url is not None:
-        package_or_url = parsed_package.valid_url
-    elif parsed_package.valid_local_path is not None:
-        package_or_url = parsed_package.valid_local_path
-
-    logging.info(f"cleaned package spec: {package_or_url}")
-
+    package_or_url = _parsed_package_to_package_or_url(
+        parsed_package, remove_version_specifiers=True
+    )
     return package_or_url
 
 
@@ -185,4 +178,30 @@ def valid_pypi_name(package_spec: str) -> Optional[str]:
         # not a valid PEP508 package specification
         return None
 
-    return package_req.name
+    if package_req.url:
+        # package name supplied by user might not match package found in URL,
+        #   so force package name determination the long way
+        return None
+
+    return canonicalize_name(package_req.name)
+
+
+def fix_package_name(package_or_url: str, package: str) -> str:
+    try:
+        package_req = Requirement(package_or_url)
+    except InvalidRequirement:
+        # not a valid PEP508 package specification
+        return package_or_url
+
+    if canonicalize_name(package_req.name) != canonicalize_name(package):
+        logging.warning(
+            textwrap.fill(
+                f"{hazard}  Name supplied in package specifier was "
+                f"{package_req.name!r} but package found has name "
+                f"{package!r}.  Using {package!r}.",
+                subsequent_indent="    ",
+            )
+        )
+    package_req.name = package
+
+    return str(package_req)
